#!/usr/bin/perl
use strict;
use XFileConfig;
use Session;

my $ses = Session->new();
my $db= $ses->db;

### Cleanup DailyTraffic ###
$db->Exec("DELETE FROM DailyTraffic WHERE dayhour<TO_DAYS(CURDATE())*24+HOUR(NOW())-6 AND bandwidth < 1024*100"); # 6 hours ago and BW<100 MB

$db->Exec("DELETE FROM DailyTraffic WHERE dayhour<TO_DAYS(CURDATE())*24+HOUR(NOW())-24*14"); # older than 14 days

# Delete old torrents
$db->Exec("DELETE FROM Torrents WHERE created<NOW() - INTERVAL 30 DAY");

# Delete old IP stats
$db->Exec("DELETE FROM StatsIP WHERE day<CURDATE()-INTERVAL 14 DAY");

# Delete old abuses
$db->Exec("DELETE FROM Reports WHERE created<NOW() - INTERVAL 3 MONTH");

$db->Exec("DELETE FROM Sessions WHERE last_time<NOW() - INTERVAL 7 DAY");

if($c->{truncate_views_daily})
{
   $db->Exec("TRUNCATE Downloads");
   $db->Exec("TRUNCATE Views");
}
elsif($c->{clean_ip2files_days})
{
   $db->Exec("DELETE FROM Downloads WHERE created<NOW() - INTERVAL ? DAY",$c->{clean_ip2files_days});
   $db->Exec("DELETE FROM Views WHERE created<NOW() - INTERVAL ? DAY",$c->{clean_ip2files_days});
   sleep 5; # take a breath
   $db->Exec("DELETE FROM Views WHERE created<NOW() - INTERVAL 24 HOUR AND finished=0"); # clean not-finished yesterday views
}

$db->Exec("DELETE FROM DelReasons WHERE last_access<NOW() - INTERVAL 6 MONTH");

# Clean old logins
$db->Exec("DELETE FROM LoginHistory WHERE created<NOW() - INTERVAL 30 DAY");

$db->Exec("DELETE FROM StatsCountry WHERE day<CURDATE() - INTERVAL 14 DAY");

$db->Exec("DELETE FROM FilesTrash WHERE file_deleted<NOW() - INTERVAL 30 DAY");

$db->Exec("DELETE FROM Pairs WHERE created<NOW() - INTERVAL 3 DAY");

$db->Exec("DELETE FROM ChangeFields WHERE created<NOW() - INTERVAL 14 DAY");

$db->Exec("DELETE FROM StatsPerf WHERE time < NOW() - INTERVAL 180 DAY");

$db->Exec("DELETE FROM StatsMisc WHERE day < NOW() - INTERVAL 365 DAY");

### Deleted files email ###
if($c->{deleted_files_reports})
{
    $c->{email_text}=1;
    my $users = $db->SelectARef("SELECT DISTINCT usr_id FROM FilesTrash");
    for my $u (@$users)
    {
       my $files = $db->SelectARef("SELECT file_name FROM FilesTrash 
                                    WHERE usr_id=?
                                    AND file_deleted>NOW()-INTERVAL 24 HOUR
                                    AND hide=0
                                    ORDER BY file_name",$u->{usr_id});
       next if $#$files==-1;
       my $user = $db->SelectRow("SELECT * FROM Users WHERE usr_id=?",$u->{usr_id});
       next unless $user;
       my $text="These files were expired or deleted by administrator from your account:\n\n";
       $text.=join("\n", map{$_->{file_name}}@$files );
       $ses->SendMail( $user->{usr_email}, $c->{email_from}, "$c->{site_name}: deleted files list", $text );
       $db->Exec("DELETE FROM FilesTrash WHERE usr_id=?",$u->{usr_id});
    }
}

# Google sitemaps
if($c->{ping_google_sitemaps})
{
      my $perfile=5000;
      print"Generating Google Sitemap...<br>\n";
      
      my @list;
      my $cx=0;
      while( my $files=$db->SelectARef("SELECT file_code,file_name FROM Files LIMIT ?,$perfile",$cx*$perfile) )
      {
         last if $#$files==-1;
         open(F, ">$c->{site_path}/upload-data/sitemap$cx.txt")||die"can't open sitemap.txt: $!";
         for(@$files)
         {
            print F $ses->makeFileLink($_),"\n";
         }
         close F;
         `gzip -c $c->{site_path}/upload-data/sitemap$cx.txt > $c->{site_path}/upload-data/sitemap$cx.txt.gz`;
         push @list, "sitemap$cx.txt.gz";
         unlink("$c->{site_path}/upload-data/sitemap$cx.txt");
         $cx++;
      }
      
      my @dd = $ses->getTime;
      my $date = "$dd[0]-$dd[1]-$dd[2]";
      open(F,">$c->{site_path}/upload-data/sitemap_index.xml")||die"can't open sitemap_index.xml: $!";
      print F qq[<?xml version="1.0" encoding="UTF-8"?>\n<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">\n];
      for(@list)
      {
        print F qq[   <sitemap>\n      <loc>$c->{site_url}/upload-data/$_</loc>\n      <lastmod>$date</lastmod>\n   </sitemap>\n];
      }
      print F qq[</sitemapindex>];
      close F;
      
      require LWP::UserAgent;
      my $ua = LWP::UserAgent->new();
      $ua->get( "http://www.google.com/webmasters/tools/ping?sitemap=".$ses->{cgi_query}->url_encode("$c->{site_url}/upload-data/sitemap_index.xml") )->content;
      $ua->get( "http://www.bing.com/ping?sitemap=".$ses->{cgi_query}->url_encode("$c->{site_url}/upload-data/sitemap_index.xml") )->content;
}