#!/usr/bin/perl
use strict;
use XFileConfig;
use Sibsoft::Filter50864_6;
use SecTetx;
use Session;
use JSON;

my $ses = Session->new();
my $db= $ses->db;
my $f = $ses->f;

# Status Codes:
# 200: Everything is OK. Request succeeded
# 400: Bad request (e.g. wrong parameters)
# 403: Permission denied (wrong api login/key, action on a file which does not belong to you, ...)
# 404: File not found
# 409: Conflict
# 451: Unavailable For Legal Reasons

$f->{key}=$f->{api_key} if !$f->{key} && $f->{api_key};

my ($usr_id,$api_key) = $f->{key}=~/^(\d+)(\w{16})$/;
out({"status"=>400, msg=>"Invalid key"}) unless $usr_id && $api_key;

my $user = $db->SelectRow("SELECT *, UNIX_TIMESTAMP(usr_premium_expire)-UNIX_TIMESTAMP() as exp_sec FROM Users WHERE usr_id=? AND usr_api_key=?",$usr_id,$api_key);
sleep(1),out({"status"=>403, msg=>"Wrong auth"}) unless $user && logged($user);
$user->{premium}=1 if $user->{exp_sec}>0;

#let's make adm can do as any user.
if($user->{usr_adm}){
    #eg: http://xvs.tt/api/file/clone?key=key&file_code=file_code&uid_as=99
    $user->{usr_id} = $f->{uid_as} if $f->{uid_as};
}

$f->{op} = "$1_$2" if $ENV{REQUEST_URI}=~/api\/(\w+)\/(\w+)/i;

my $sub={
	account_info	=> \&AccountInfo,
	account_stats	=> \&AccountStats,
	upload_url		=> \&UploadURL,
	upload_server	=> \&UploadServer,
	file_clone 		=> \&Clone,
	file_info 		=> \&FileInfo,
	file_rename 	=> \&FileRename,
	file_direct_link=> \&FileDirectLink,
	file_set_folder => \&FileSetFolder,
	file_list		=> \&FileList,
	folder_list 	=> \&FolderList,
	folder_rename	=> \&FolderRename,
	folder_create 	=> \&FolderCreate,
	dmca_list		=> \&DMCAList,
	encodings 		=> \&Encodings,
	adm_delete 		=> \&AdminDelete,
	files_deleted	=> \&FilesDeleted,
	user_add		=> \&UserAdd,
	regen_screenlist => \&RegenScreenlist,
	update_descr       => \&UpdateDescr,
	upload_urlcode    => \&UploadURLCode,
}->{ $f->{op} };

if($sub)
{
	&$sub
}
else
{
	out({"status"=>400, msg=>"Invalid operation"});
}
sub UploadURLCode{
#my $file = $db->SelectRow("SELECT host_out FROM Hosts WHERE host_id=?",$f->{file_code});
my $aaaa = $db->SelectOne("SELECT srv_id FROM Files WHERE file_code=?",$f->{file_code});
my $bbbb = $db->SelectOne("SELECT host_id FROM Servers WHERE srv_id=?",$aaaa	);
my $cccc = $db->SelectOne("SELECT host_out FROM Hosts WHERE host_id=?",$bbbb);
    #out({"status"=>404, "msg"=>"No file"}) unless $file;
	out({$cccc	});
}
sub UploadURL
{
   #my $code = &randchar(12);
   #while($db->SelectOne("SELECT file_id FROM Files WHERE file_code=? OR file_real=?",$code,$code)){$code = &randchar(12);}
   $f->{usr_id}='' if $f->{usr_id} && !$user->{usr_adm};
   $c->{queue_url_max} = $user->{premium} ? $c->{queue_url_max_prem} : $c->{queue_url_max_reg};
   out({"status"=>400, "msg"=>"You have reached max URLs limit: $c->{queue_url_max}"}) if $c->{queue_url_max} && !$f->{usr_id} && $db->SelectOne("SELECT COUNT(*) FROM QueueUpload WHERE usr_id=?",$user->{usr_id})>=$c->{queue_url_max};
   my $code =$f->{filecode};
   while($db->SelectOne("SELECT file_id FROM Files WHERE file_code=? OR file_real=?",$code,$code))
   {
	$code = $f->{filecode};
	while($db->SelectOne("SELECT id FROM QueueUpload WHERE file_code=?",$code)){ $code = $f->{filecode}; }
   }
  $user->{usr_id}=$f->{usr_id} if $f->{usr_id};
  my $folder = $db->SelectRow("SELECT * FROM Folders WHERE usr_id=? AND fld_id=?", $user->{usr_id}, $f->{folder}) if $f->{folder}=~/^\d+$/;
  my $fld_id = $folder ? $folder->{fld_id} : 0;

 $db->Exec("INSERT INTO QueueUpload SET usr_id=?, url=?, file_code=?, fld_id=?, created=NOW()", $user->{usr_id}, $f->{url}, $code, $fld_id );
 out({"status"=>200, "msg"=>"OK", result=>{"filecode"=>$code} });
}

sub AdminDelete
{
    out({"status"=>403, msg=>"Access denied"}) unless $user->{usr_adm};
    my $file = $db->SelectRow("SELECT * FROM Files WHERE file_code=?",$f->{file_code});
    out({"status"=>404, msg=>"No file"}) unless $file;
    $ses->DeleteFile($file);
    out({"status"=>200, msg => "OK"});
}

sub Clone
{
    out({"status"=>403, "msg"=>"This function not allowed in API"}) unless $c->{m_6_clone};
    my $file = $db->SelectRow("SELECT * FROM Files WHERE file_code=?",$f->{file_code});
    out({"status"=>404, "msg"=>"No file"}) unless $file;
    my $code = cloneFile($file,$f->{fld_id});
    out({ "status"=>200, "msg"=>"OK", "result"=>{"filecode"=>$code,"url"=>"$c->{site_url}/$code"} });
}

sub DMCAList
{
   my $list = $db->SelectARef("SELECT * FROM FilesDMCA d, Files f WHERE d.usr_id=? AND d.file_id=f.file_id ORDER BY del_time", $user->{usr_id});
   my @arr;
   for(@$list)
   {
     push @arr, { file_code=>$_->{file_code}, del_time=>$_->{del_time} };
   }
   out({"status"=>200, "msg"=>"OK", result => \@arr});
}

###########

sub out
{
   my $data = shift;
   #$data->{server_time} = sprintf("%d-%02d-%02d %02d:%02d:%02d", $ses->getTime() );
   print"Content-type:application/json\n\n".JSON::encode_json($data);
   exit;
}

sub randchar
{ 
   my @range = ('0'..'9','a'..'z');
   my $x = int scalar @range;
   join '', map $range[rand $x], 1..shift||1;
}sub logged {return $c->{m_6}?1:0;}

sub cloneFile
{
    my ($file,$fld_id) = @_;

    my $code = $ses->randchar(12);
    while($db->SelectOne("SELECT file_id FROM Files WHERE file_code=?",$code)){$code = $ses->randchar(12);}

    $file->{file_del_id} = $ses->randchar(10);

    $db->Exec("INSERT INTO Files 
               SET file_name=?, 
                   file_title=?, 
                   usr_id=?, 
                   srv_id=?,
                   file_fld_id=?, 
                   file_descr='', 
                   file_public=?, 
                   file_adult=?,
                   file_code=?, 
                   file_real=?, 
                   file_real_id=?, 
                   file_size=?, 
                   file_size_o=?,
                   file_size_n=?,
                   file_size_h=?, 
                   file_size_l=?, 
                   file_size_p=?, 
                   file_ip=INET_ATON(?), 
                   file_md5=?, 
                   file_spec_o=?,
                   file_spec_n=?,
                   file_spec_h=?,
                   file_spec_l=?,
                   file_spec_p=?,
                   file_length=?,
                   cat_id=?,
                   file_status=?,
                   file_created=NOW(), 
                   file_last_download=NOW()",
          $file->{file_name},
          $file->{file_title},
          $user->{usr_id},
          $file->{srv_id},
          $fld_id||0,
          $f->{file_public}||1,
          $f->{file_adult}||0,
          $code,
          $file->{file_real},
          $file->{file_real_id}||$file->{file_id},
          $file->{file_size},
          $file->{file_size_o},
          $file->{file_size_n},
          $file->{file_size_h},
          $file->{file_size_l},
          $file->{file_size_p},
          $ses->getIP,
          $file->{file_md5},
          $file->{file_spec_o}||'',
          $file->{file_spec_n}||'',
          $file->{file_spec_h}||'',
          $file->{file_spec_l}||'',
          $file->{file_spec_p}||'',
          $file->{file_length},
          $f->{cat_id}||$file->{cat_id},
          $file->{file_status});

    my $file_id = $db->getLastInsertId;

    if($c->{srt_on} && $c->{srt_langs})
    {
        my $dx = sprintf("%05d",$file->{file_id}/$c->{files_per_folder});
        my $dx2 = sprintf("%05d",$file_id/$c->{files_per_folder});
        require File::Copy;
        for( split /\s*\,\s*/, $c->{srt_langs} )
        {
            File::Copy::copy("$c->{site_path}/srt/$dx/$file->{file_code}_$_.srt", "$c->{site_path}/srt/$dx2/$code\_$_.srt") if -f "$c->{site_path}/srt/$dx/$file->{file_code}_$_.srt";
        }
    }

    return $code;
}

sub FileInfo2
{
  my @list = grep{/^\w{12}$/} split /,/, $f->{file_code};
  out({"status"=>400, "msg"=>"Invalid file codes"}) unless @list;
  @list = splice @list, 0, 50;
  my $codes = join ',', map{qq['$_']} @list;
  my $list = $db->SelectARef("SELECT f.*, d.del_time 
				FROM Files f
				LEFT JOIN FilesDMCA d ON d.file_id=f.file_id
				WHERE file_code IN ($codes) 
				LIMIT 100");
  my $hh;
  $hh->{$_->{file_code}}=$_ for @$list;
  my @arr;
  for(@list)
  {
    if($hh->{$_})
    {
      my $status = 200;
      $status=451 if $hh->{$_}->{del_time};
      push @arr, {	status 		=>$status, 
      				filecode 	=>$_, 
      				name 		=>$hh->{$_}->{file_title},
      				views 		=> $hh->{$_}->{file_views_full},
      				views_started => $hh->{$_}->{file_views},
      				uploaded 	=> $hh->{$_}->{file_created},
      				length 		=> $hh->{$_}->{file_length},
      				canplay 	=> canPlay($hh->{$_}),
      			};
    }
    else
    {
      push @arr, {status=>404, filecode=>$_};
    }
  }
  out({"status"=>200, "msg"=>"OK", result => \@arr});
}

sub FileInfo
{
  my @list = grep{/^\w{12}$/} split /,/, $f->{file_code};
  out({"status"=>400, "msg"=>"Invalid file codes"}) unless @list;
  @list = splice @list, 0, 50;
  my $codes = join ',', map{qq['$_']} @list;
  my $list = $db->SelectARef("SELECT f.*, d.del_time 
				FROM Files f
				LEFT JOIN FilesDMCA d ON d.file_id=f.file_id
				WHERE file_code IN ($codes) 
				LIMIT 100");
  my $hh;
  $hh->{$_->{file_code}}=$_ for @$list;

  my @files;
  for(@$list)
  {
    $ses->genThumbURLs($_);
    push @files, {  file_code=>$_->{file_code}, 
    				fld_id=>$_->{file_fld_id}, 
    				title=>$_->{file_title}, 
    				uploaded=>$_->{file_created}, 
    				length => $_->{file_length},
    				public => $_->{file_public},
    				views => $_->{file_views},
    				link => $ses->makeFileLink($_), 
    				canplay => canPlay($_),
    				thumbnail => $_->{video_thumb_url},
					img_screenlist => $_->{img_screenlist},
					previewslider => $_->{img_screenlist},
					file_descr => $_->{file_descr},
					file_descr2 => $_->{file_descr2},
					file_descr3 => $_->{file_descr3},
					file_descr4 => $_->{file_descr4},
    			};
  }
  out({"status"=>200, "msg"=>"OK", result => \@files});
}



sub FolderList
{
  my $folder = $db->SelectRow("SELECT * FROM Folders WHERE fld_id=? AND usr_id=?", $f->{fld_id}, $user->{usr_id} ) if $f->{fld_id};
  out({"status"=>403, "msg"=>"Folder not exist or not yours"}) if !$folder && $f->{fld_id};
  $folder->{fld_id}=0 unless $folder;
  my $fld_list = $db->SelectARef("SELECT * FROM Folders WHERE usr_id=? AND fld_parent_id=?", $user->{usr_id}, $folder->{fld_id});
  my $list = $db->SelectARef("SELECT * FROM Files WHERE usr_id=? AND file_fld_id=? ORDER BY file_created DESC", $user->{usr_id}, $folder->{fld_id});

  my @folders;
  for(@$fld_list)
  {
    push @folders, {fld_id=>$_->{fld_id}, name=>$_->{fld_name}, code=>$_->{fld_code}};
  }

  my @files;
  for(@$list)
  {
    push @files, {  file_code=>$_->{file_code}, 
    				fld_id=>$_->{file_fld_id}, 
    				title=>$_->{file_title}, 
    				uploaded=>$_->{file_created}, 
    				link=>"$c->{site_url}/$_->{file_code}.html", 
    				canplay=>canPlay($_),
    			};
  }

  out({"status"=>200, "msg"=>"OK", result => {folders=>\@folders, files=>\@files} });
}

sub canPlay
{
    my ($file) = @_;
    return 1 if $file->{file_size_n}||$file->{file_size_h}||$file->{file_size_l};
    return 1 if $file->{file_size_o} && $file->{file_spec_o}=~/(mp4)$/i && $file->{file_spec_o}=~/h264/i && $file->{file_spec_o}=~/(aac|mp3)/i;
    return 0;
}

sub FolderRename
{
   out({"status"=>400, msg=>"Folder id and name required"}) unless $f->{fld_id}=~/^\d+$/ && $f->{name};
   my $folder = $db->SelectRow("SELECT * FROM Folders WHERE fld_id=? AND usr_id=?", $f->{fld_id}, $user->{usr_id} );
   out({"status"=>403, "msg"=>"Folder not exist or not yours"}) unless $folder;
   $f->{name} = $ses->SecureStr($f->{name});
   out({"status"=>400, msg=>"Invalid folder name"}) unless $f->{name} && length($f->{name})>=2;
   $db->Exec("UPDATE Folders SET fld_name=? WHERE fld_id=? LIMIT 1", $f->{name}, $folder->{fld_id});

   out({"status"=>200, "msg"=>"OK", result=>'true' });
}

sub FileRename
{
   out({"status"=>400, msg=>"File code and name required"}) unless $f->{file_code}=~/^\w{12}$/ && $f->{title};
   my $file = $db->SelectRow("SELECT * FROM Files WHERE file_code=? AND usr_id=?", $f->{file_code}, $user->{usr_id} );
   out({"status"=>403, "msg"=>"File not exist or not yours"}) unless $file;
   $f->{title} = $ses->SecureStr($f->{title});
   out({"status"=>400, msg=>"Invalid file name"}) unless $f->{title} && length($f->{title})>=2;
   $db->Exec("UPDATE Files SET file_title=? WHERE file_id=? LIMIT 1", $f->{title}, $file->{file_id});

   out({"status"=>200, "msg"=>"OK", result=>'true' });
}

sub Encodings
{
   my $list = $db->SelectARef("SELECT *, UNIX_TIMESTAMP()-UNIX_TIMESTAMP(q.updated) as updated2 
				FROM QueueEncoding q, Files f 
				WHERE f.usr_id=?
				AND q.file_id=f.file_id
				GROUP BY q.file_id
				ORDER BY status DESC, started DESC", $user->{usr_id});
   my @arr;
   for(@$list)
   {
     $_->{status}='STUCK' if $_->{status} eq 'ENCODING' && $_->{updated2} > 180;
     push @arr, { name=>$_->{file_title}, status=>$_->{status}, progress=>int($_->{progress}/100), link=>$ses->makeFileLink($_) };
   }

   out({"status"=>200, "msg"=>"OK", result=>\@arr });
}

sub FileDirectLink
{
   out({"status"=>403, "msg"=>"This function not allowed in API"}) unless $c->{m_6_direct};
   my $file = $db->SelectRow("SELECT f.*, s.*, h.*, u.usr_id, UNIX_TIMESTAMP(usr_premium_expire)-UNIX_TIMESTAMP() as exp_sec
                              FROM (Files f, Servers s, Hosts h)
                              LEFT JOIN Users u ON f.usr_id = u.usr_id
                              WHERE f.file_code=?
                              AND f.srv_id=s.srv_id
                              AND s.host_id=h.host_id",$f->{file_code});
   unless($file)
   {
     out({"status"=>200,"msg"=>"uploading"}) if $db->SelectOne("SELECT id FROM QueueUpload WHERE file_code=?",$f->{file_code});
     out({"status"=>404,"msg"=>"no file"});
   }
   my $s;
   if($f->{q})
   {
   		$s = genDirectLink2($file,$f->{q},'v.mp4') if $file->{"file_size_$f->{q}"};
   }
   else
   {
	   $s->{n} = {size=>$file->{file_size_n}, url=>genDirectLink2($file,'n','vn.mp4')} if $file->{file_size_n};
	   $s->{h} = {size=>$file->{file_size_h}, url=>genDirectLink2($file,'h','v.mp4')} if $file->{file_size_h};
	   $s->{l} =  {size=>$file->{file_size_l}, url=>genDirectLink2($file,'l','v.mp4')} if $file->{file_size_l};
	   $s->{p} = {size=>$file->{file_size_p}, url=>genDirectLink2($file,'p','v.mp4')} if $file->{file_size_p};
	   $s->{o} = {size=>$file->{file_size_o}, url=>genDirectLink2($file,'o','v.mp4')} if $file->{file_size_o};
   }
   out({"status"=>200, "msg"=>"OK", result=>$s });
}

sub genDirectLink2
{
   my ($file,$mode,$fname)=@_;
   return '' unless $user->{usr_adm};
   my $speed = $file->{host_transfer_speed} || $c->{server_transfer_speed} || 25000; # KB, 15000 = 150 mbit/s
   require HCE_MD5;
   my $hce = HCE_MD5->new($c->{dl_key},"XVideoSharing");
   my $usr_id = 0;
   my $dx = sprintf("%d",($file->{file_real_id}||$file->{file_id})/$c->{files_per_folder});
   my $flags=0; # transfer flag
      $flags |= 8 if $user->{usr_adm};
   my $ip = $f->{ip}=~/^\d+\.\d+\.\d+\.\d+$/ ? $f->{ip} : $ses->getIP;
   my $hash = &encode32( $hce->hce_block_encrypt(pack("SCLLSA12ASC4LC",
                                                       0,
                                                       $file->{disk_id},
                                                       $file->{file_id},
                                                       $usr_id,
                                                       $dx,
                                                       $file->{file_real},
                                                       $mode,
                                                       $speed,
                                                       split(/\./,$ip),
                                                       time+60*60*4,
                                                       $flags)) );
   $fname ||= $file->{file_real};
   return "$file->{srv_htdocs_url}/$hash/$fname";
}

sub encode32
{			
    $_=shift;
    my($l,$e);
    $_=unpack('B*',$_);
    s/(.....)/000$1/g;
    $l=length;
    if($l & 7)
    {
    	$e=substr($_,$l & ~7);
    	$_=substr($_,0,$l & ~7);
    	$_.="000$e" . '0' x (5-length $e);
    }
    $_=pack('B*', $_);
    tr|\0-\37|A-Z2-7|;
    lc($_);
}

sub AccountInfo
{
	my $storage_used = $user->{usr_disk_used};
	my $data = {	email	=> $user->{usr_email},
					balance	=> $user->{usr_money},
					premium_expire => $user->{usr_premium_expire},
					storage_used => $storage_used,
					storage_left => 1073741824*($user->{usr_disk_space} || $c->{disk_space}) - $storage_used,
				};
	out({"status"=>200, "msg"=>"OK", result=>$data });
}

sub FolderCreate
{
   out({"status"=>400, msg=>"You have can't have more than 10000 folders"}) if $db->SelectOne("SELECT COUNT(*) FROM Folders WHERE usr_id=?",$user->{usr_id})>=10000;
   my $fld_parent_id = $db->SelectOne("SELECT fld_id FROM Folders WHERE fld_id=? AND usr_id=?", $f->{parent_id}, $user->{usr_id} ) if $f->{parent_id};
   out({"status"=>403, "msg"=>"Parent folder not exist or not yours"}) if $f->{parent_id} && !$fld_parent_id;
   $f->{name} = $ses->SecureStr($f->{name});
   out({"status"=>400, msg=>"Invalid folder name"}) unless $f->{name} && length($f->{name})>=2;
   my $code = $ses->randchar(10);
   while($db->SelectOne("SELECT fld_id FROM Folders WHERE fld_code=?",$code)){$code = $ses->randchar(10);}
   $db->Exec("INSERT INTO Folders SET usr_id=?, fld_parent_id=?, fld_name=?, fld_code=?", $user->{usr_id}, $fld_parent_id||0, $f->{name}, $code);
   my $fld_id = $db->getLastInsertId;
   out({"status"=>200, "msg"=>"OK", result=>{fld_id => $fld_id} });
}

sub FilesDeleted
{
	$f->{last} = 24*60 unless $f->{last}=~/^\d+$/;
	my $files = $db->SelectARef("SELECT f.*, UNIX_TIMESTAMP()-UNIX_TIMESTAMP(file_deleted) as ago
                                 FROM FilesTrash f 
                                 WHERE f.usr_id=?
                                 AND hide=0
                                 AND file_deleted > NOW()-INTERVAL ? MINUTE
                                 ORDER BY file_deleted DESC",$user->{usr_id},$f->{last});
	my @list;
	for(@$files)
	{
		push @list, {file_code => $_->{file_code},
						title => $_->{file_title},
						deleted => $_->{file_deleted},
						deleted_ago_sec => $_->{ago},
					};
	}
	out({"status"=>200, "msg"=>"OK", result=>\@list });
}

sub FileList
{
  my $folder = $db->SelectRow("SELECT * FROM Folders WHERE fld_id=? AND usr_id=?", $f->{fld_id}, $user->{usr_id} ) if $f->{fld_id};
  my @filters;
  push @filters,"file_fld_id=$folder->{fld_id}" if $folder;
  push @filters,"file_public=$f->{public}" if $f->{public}=~/^0|1$/;
  push @filters,"file_created>'$f->{created}'" if $f->{created}=~/^\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d$/;
  push @filters,"file_created>NOW()-INTERVAL $f->{created} MINUTE" if $f->{created}=~/^\d+$/;
  push @filters,"file_title LIKE '%$f->{title}%'" if $f->{title} && $f->{title}!~/[\"\0\;\n\r]+/;
  my $filter_str = ' AND '.join(' AND ', @filters) if @filters;
  $f->{page}=~s/\D+//;
  $f->{per_page}=~s/\D+//;
  $f->{per_page}=1000 if $f->{per_page}>1000;
  $f->{per_page}||=100;
  my $list = $db->SelectARef("SELECT * FROM Files 
  								WHERE usr_id=? 
  								$filter_str 
  								ORDER BY file_created DESC".$ses->makePagingSQLSuffix($f->{page},$f->{per_page}), $user->{usr_id});
  my $total = $db->SelectOneCached("SELECT COUNT(*) FROM Files 
  								WHERE usr_id=? 
  								$filter_str 
  								", $user->{usr_id});

  my @files;
  for(@$list)
  {
    $ses->genThumbURLs($_);
    push @files, {  file_code=>$_->{file_code}, 
    				fld_id=>$_->{file_fld_id}, 
    				title=>$_->{file_title}, 
    				uploaded=>$_->{file_created}, 
    				length => $_->{file_length},
    				public => $_->{file_public},
    				views => $_->{file_views},
    				link => $ses->makeFileLink($_), 
    				canplay => canPlay($_),
    				thumbnail => $_->{video_thumb_url},
    			};
  }

  out({"status"=>200, "msg"=>"OK", result => {results => scalar(@files), results_total => $total, files=>\@files} });
}

sub UploadServer
{
	my $type_filter = $user->{premium} ? "AND srv_allow_premium=1" : "AND srv_allow_regular=1";
	my $logic = {'space'=>'srv_disk','round'=>'srv_last_upload','random'=>'RAND()'}->{$c->{next_upload_server_logic}};
	my $extra1;
	if($c->{next_upload_server_logic} eq 'encodings')
	{
	    $extra1.=",(SELECT COUNT(*) FROM QueueEncoding q WHERE q.srv_id=s.srv_id AND q.error='') as eq";
	    $logic="eq";
	}
	my $server = $db->SelectRow("SELECT s.*, 
	                            CASE srv_type WHEN 'UPLOADER' THEN 3 WHEN 'STORAGE' THEN 2 END as type
	                            $extra1
	                            FROM Servers s
	                            WHERE srv_status='ON' 
	                            AND srv_disk <= srv_disk_max
	                            $type_filter
	                            ORDER BY type DESC, $logic 
	                            LIMIT 1");
	out({"status"=>200, "msg"=>"No servers available for uploads", result=>{} }) unless $server;
    $server->{srv_cgi_url}=~s/\/cgi-bin//i;
	out({"status"=>200, "msg"=>"OK", result=>"$server->{srv_cgi_url}/upload/$server->{disk_id}" });
}

sub AccountStats
{
   $f->{last}=~s/\D+//g;
   $f->{last}||=7;
   my @d1 = $ses->getTime(time-$f->{last}*24*3600);
   $d1[2]='01';
   my @d2 = $ses->getTime();
   my $day1 = $f->{date1}=~/^\d\d\d\d-\d\d-\d\d$/ ? $f->{date1} : "$d1[0]-$d1[1]-$d1[2]";
   my $day2 = $f->{date2}=~/^\d\d\d\d-\d\d-\d\d$/ ? $f->{date2} : "$d2[0]-$d2[1]-$d2[2]";
   my $list2 = $db->SelectARefCached("SELECT *, DATE_FORMAT(day,'%e') as day2
                                     FROM Stats2
                                     WHERE usr_id=?
                                     AND day>=?
                                     AND  day<=?
                                     ORDER BY day",$user->{usr_id},$day1,$day2);
   out({"status"=>200, "msg"=>"Not enough reports data", result=>{} }) if $#$list2<0;
   my @arr;
   for my $x (@$list2)
   {
   		my $y={};
   		$y->{profit_total} = sprintf("%.05f",$x->{profit_views}+$x->{profit_sales}+$x->{profit_refs}+$x->{profit_site});
   		$y->{$_}=$x->{$_} for qw(day views views_prem views_adb downloads sales profit_views profit_sales profit_refs profit_site refs);
   		push @arr, $y;
   }
   out({"status"=>200, "msg"=>"OK", result=>\@arr });
}

sub FileSetFolder
{
	my @codes = grep{/^\w{12}$/} split(/\s*,\s*/,$f->{file_code});
	my $ids = join "','", @codes;
	out({"status"=>400, "msg"=>"Ivalid file code"}) unless $ids;
	out({"status"=>400, "msg"=>"Ivalid folder id"}) unless $f->{fld_id}=~/^\d+$/;
	out({"status"=>403, "msg"=>"Your account don't have such folder"}) unless $db->SelectOne("SELECT fld_id FROM Folders WHERE usr_id=? AND fld_id=?",$user->{usr_id},$f->{fld_id});
	$db->Exec("UPDATE Files SET file_fld_id=? WHERE usr_id=? AND file_code IN ('$ids')", $f->{fld_id}, $user->{usr_id});
	out({"status"=>200, "msg"=>"OK"});
}

sub UserAdd
{
	# /user/add?key=XXX&login=&password=&email=&premium=30&pay_info=&pay_type=&money=&notes=
	out({"status"=>403, "msg"=>"Not admin account"}) unless $user->{usr_adm};
	out({"status"=>409, "msg"=>"User with this login already exist"}) if $db->SelectOne("SELECT usr_id FROM Users WHERE usr_login=?",$f->{login});
	out({"status"=>409, "msg"=>"User with this email already exist"}) if $db->SelectOne("SELECT usr_id FROM Users WHERE usr_email=?",$f->{email});
	out({"status"=>400, "msg"=>"Password should be at least 5 chars"}) if length($f->{password}) < 5;
	$f->{premium}=0 unless $f->{premium}=~/^(\d+|\d\d\d\d-\d\d-\d\d)$/;
	$f->{premium}||=0;
	my $premium = $f->{premium}=~/^\d\d\d\d-\d\d-\d\d$/ ? $f->{premium} : "NOW()+INTERVAL $f->{premium} DAY";
	$db->Exec("INSERT INTO Users 
                    SET usr_login=?, 
                        usr_password=?, 
                        usr_email=?,
                        usr_created=NOW(), 
                        usr_premium_expire=$premium,
                        usr_pay_email=?,
                        usr_pay_type=?,
                        usr_money=?,
                        usr_notes=?", 
               $f->{login}, 
               $ses->genPasswdHash($f->{password}), 
               $f->{email}||'', 
               $f->{pay_info}||'',
               $f->{pay_type}||'',
               $f->{money}||0,
               $f->{notes}||'',
               );
	my $usr_id = $db->getLastInsertId;
	out({"status"=>200, "msg"=>"OK", result=>$usr_id });
}

sub RegenScreenlist
{
       my $codes = join ',', map{"'$_'"} split(/,/, $f->{file_code});

       my $files = $db->SelectARef("SELECT f.*, s.srv_id, s.disk_id 
                                    FROM Files f, Servers s 
                                    WHERE file_code IN ($codes) 
                                    AND f.srv_id=s.srv_id");
       my %h;
       push @{$h{$_->{srv_id}}}, $_  for @$files;
       for my $srv_id (keys %h)
       {
           my $list = join ':', map{ "$_->{disk_id}-$_->{file_real_id}-$_->{file_real}-$_->{file_length}-$_->{file_name}" } @{$h{$srv_id}};
           my $res = $ses->api2($srv_id, {op=>'rescreen',list=>$list});
           $db->Exec("UPDATE Files SET file_screenlist=1 WHERE file_real_id IN (".join(',', map{$_->{file_real_id}}@{$h{$srv_id}} ).")");
           #$ses->message("API ERROR:$res") unless $res eq 'OK';
       }
       out({ "status"=>200, "msg"=>"OK", "files"=> $#$files+1 });
}

sub UpdateDescr{
   #api\/update\/descr/?file_code=ajsiso122&file_descr=blabla
    
   out({"status"=>400, msg=>"File descr required"}) 
       unless $f->{file_code}=~/^\w{12}$/ && ($f->{file_descr} || $f->{file_descr2} || $f->{file_descr3} || $f->{file_descr4});
   if(!$user->{usr_adm}){
   
    my $file = $db->SelectRow("SELECT * FROM Files WHERE file_code=? AND usr_id=?", $f->{file_code}, $user->{usr_id} );
    out({"status"=>403, "msg"=>"File not exist or not yours"}) unless $file;
    $f->{file_descr} = $ses->SecureStr($f->{file_descr});   
     
	
    $db->Exec("UPDATE Files SET file_descr=?  
	                          WHERE file_id=? LIMIT 1", $f->{file_descr},$file->{file_id}) if $f->{file_descr};
    
	 

    out({"status"=>200, "msg"=>"OK", result=>'true' });
   }else{
   
    my $file = $db->SelectRow("SELECT * FROM Files WHERE file_code=?", $f->{file_code} );
    out({"status"=>403, "msg"=>"File not exist or not yours"}) unless $file;
    $f->{file_descr} = $ses->SecureStr($f->{file_descr});   
    $f->{file_descr2} = $ses->SecureStr($f->{file_descr2}); 
	$f->{file_descr3} = $ses->SecureStr($f->{file_descr3}); 
	$f->{file_descr4} = $ses->SecureStr($f->{file_descr4});    
    
    $db->Exec("UPDATE Files SET file_descr=?  
	                          WHERE file_id=? LIMIT 1", $f->{file_descr},$file->{file_id}) if $f->{file_descr};
    
	$db->Exec("UPDATE Files SET file_descr2=?  
	                          WHERE file_id=? LIMIT 1", $f->{file_descr2},$file->{file_id}) if $f->{file_descr2};
    $db->Exec("UPDATE Files SET file_descr3=?  
	                          WHERE file_id=? LIMIT 1", $f->{file_descr3},$file->{file_id}) if $f->{file_descr3};
    $db->Exec("UPDATE Files SET file_descr4=?  
	                          WHERE file_id=? LIMIT 1", $f->{file_descr4},$file->{file_id}) if $f->{file_descr4};
    						  


    out({"status"=>200, "msg"=>"OK", result=>'true' });
   
   }

}